/*
 * $Id: TTY.java,v 1.3 2002-09-01 15:05:12-07 ball Exp $
 *
 * Copyright 2002 Allen D. Ball.  All rights reserved.
 */
package gui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;

/**
 * TTY extends the JTextArea to provide a Swing componeent which may
 * serve as a terminal emulator.
 *
 * @author <a href="mailto:ball@iprotium.com">Allen D. Ball</a>
 * @version $Revision: 1.3 $
 */
public class TTY extends JTextArea {
    private static final int defaultRows =       24;
    private static final int defaultColumns =    80;
    private static final int defaultBufferSize = 4096;

    private int bufferSize = defaultBufferSize;
    private InputStream in =  new TTYInputStream();
    private PrintStream out = new PrintStream(new TTYOutputStream(), true);
    private PrintStream err = new PrintStream(new TTYOutputStream(), true);

    /**
     * Creates a TTY with the argument number of rows and columns.
     *
     * @param   rows            The number of rows the terminal
     *                          emulator should have.
     * @param   columns         The number of columns the terminal
     *                          emulator should have.
     * @param   bufferSize      The number of characters to buffer in
     *                          the TTY.
     */
    public TTY(int rows, int columns, int bufferSize) {
        super(rows, columns);
        this.bufferSize = bufferSize;
        setEditable(false);
    }

    /**
     * Creates a TTY with the argument number of rows and columns.
     *
     * @param   rows            The number of rows the terminal
     *                          emulator should have.
     * @param   columns         The number of columns the terminal
     *                          emulator should have.
     */
    public TTY(int rows, int columns) {
        this(rows, columns, defaultBufferSize);
    }

    /**
     * Creates a TTY with the default number of rows and columns
     * (24x80).
     */
    public TTY() { this(defaultRows, defaultColumns); }

    /**
     * Method to get the input stream.  Characters typed to the TTY
     * may be read on the input stream.
     *
     * <p>Note: The read method of the input stream returned by this
     * method will return -1 when first called and will throw
     * java.io.EOFException on each subsequent call.</p>
     *
     * @return                  The terminal emulator's input stream.
     */
    public synchronized InputStream getIn() { return in; }

    /**
     * Method to get the output stream.  Characters written to the
     * output stream are displayed on the TTY.
     *
     * @return                  The terminal emulator's output stream.
     */
    public synchronized PrintStream getOut() { return out; }

    /**
     * Method to get the error stream.  Characters written to the
     * error stream are displayed on the TTY.
     *
     * @return                  The terminal emulator's error stream.
     */
    public synchronized PrintStream getErr() { return err; }

    private class TTYInputStream extends InputStream {
        private byte[] buf = new byte[64];
        private int pos = 0;
        private int len = 0;
        private boolean eof = false;

        public synchronized int read() throws IOException {
            if (eof) {
                throw new EOFException();
            }

            int b = -1;         // Tacky !!!

            if (b < 0) {
                eof = true;
            }

            return b;
        }
    }

    private class TTYOutputStream extends OutputStream {
        private byte[] buf = new byte[512];
        private int pos = 0;
        private int len = 0;

        public synchronized void write(int b) {
            if (len >= buf.length) {
                flush();
            }

            buf[len++] = (byte) b;
        }

        public synchronized void flush() {
            append(new String(buf, pos, len));
            setCaretPosition(getText().length());

            pos = 0;
            len = 0;

            if (getText().length() > bufferSize) {
                replaceRange("", 0, getText().length() - bufferSize);
            }
        }
    }
}
/*
 * $Log: TTY.java,v $
 * Revision 1.3  2002-09-01 15:05:12-07  ball
 * Moved package because it should be able to display both world and player
 * stuff.
 *
 * Revision 1.2  2002-09-01 08:50:25-07  ball
 * Limit size of TTY buffer.
 * Set table preferred widths when assigning a new model.
 *
 * Revision 1.1  2002-08-31 17:55:45-07  ball
 * Initial revision
 *
 */
