/*
 * $Id: Connection.java,v 1.3 2002-09-01 22:02:34-07 ball Exp $
 *
 * Copyright 2002 Allen D. Ball.  All rights reserved.
 */
package protocol;

import java.io.*;
import java.net.*;
import player.*;
import world.*;

/**
 * Class to manage the connection between a player and the ICFP-2002
 * programming server.
 *
 * @author <a href="mailto:ball@iprotium.com">Allen D. Ball</a>
 * @version $Revision: 1.3 $
 */
public class Connection {
    static private final String NL = "\n";

    private Socket socket = null;
    private BufferedReader reader = null;
    private PrintWriter writer = null;
    private PrintStream tracePrintStream = null;

    /**
     * Creates a connection between a player and the game server (on the
     * argument socket).
     *
     * @param   socket          The socket connecting the player to the
     *                          game server.
     * @param   trace           True if communications with the server
     *                          should be traced
     */
    public Connection(Socket socket, PrintStream tracePrintStream) {
        this.socket = socket;
        this.tracePrintStream = tracePrintStream;
    }

    /**
     * Creates a connection between a player and the game server (on the
     * argument socket).
     *
     * @param   socket          The socket connecting the player to the
     *                          game server.
     */
    public Connection(Socket socket) { this(socket, null); }

    private Connection() { }

    /**
     * Method to get the {@link Socket} used for communicating to the
     * game server on this connection.
     *
     * @return                  The socket for this connection.
     */
    public synchronized Socket getSocket() { return socket; }

    /**
     * Method to get a reader for the connection socket.
     *
     * @return                  A reader for the socket.
     *
     * @throws  IOException     Thrown if an exception is encountered
     *                          communicating on the connection.
     */
    public synchronized BufferedReader getReader() throws IOException {
        if (reader == null) {
            InputStream inputStream = socket.getInputStream();
            reader = new BufferedReader(new InputStreamReader(inputStream));
        }

        return reader;
    }

    /**
     * Read a line of text from the game server discarding line
     * terminator(s).
     *
     * @return                  A String containing the contents of the
     *                          line, not including any line-termination
     *                          characters.
     *
     * @throws  IOException     Thrown if an exception is encountered
     *                          communicating on the connection.
     */
    public String read() throws IOException {
        String string = getReader().readLine();

        if (string == null) {
            trace("--> " + "<EOF>");
            throw new EOFException();
        }

        trace("--> " + string);

        return string;
    }

    /**
     * Method to get a writer for the connection socket.
     *
     * @return                  A print-writer for the socket.
     *
     * @throws  IOException     Thrown if an exception is encountered
     *                          communicating on the connection.
     */
    protected synchronized PrintWriter getWriter() throws IOException {
        if (writer == null) {
            writer = new PrintWriter(socket.getOutputStream(), true);
        }

        return writer;
    }

    /**
     * Sends the argument string to the server with the proper line
     * terminator(s).
     *
     * @param   string          The string to be sent to the game
     *                          server.
     *
     * @throws  IOException     Thrown if an exception is encountered
     *                          communicating on the connection.
     */
    public synchronized void send(String string) throws IOException {
        trace("<-- " + string);
        getWriter().print(string + NL);
        getWriter().flush();
    }

    /**
     * Sends the argument object (by invoking its
     * {@link Object#toString} method) to the server with the proper
     * line terminator(s).
     *
     * @param   object          The object to be sent to the game
     *                          server.
     *
     * @throws  IOException     Thrown if an exception is encountered
     *                          communicating on the connection.
     */
    public synchronized void send(Object object) throws IOException {
        send(object.toString());
    }

    private synchronized void trace(String string) throws IOException {
        if (tracePrintStream != null) {
            tracePrintStream.println(string);
        }
    }
}
/*
 * $Log: Connection.java,v $
 * Revision 1.3  2002-09-01 22:02:34-07  ball
 * Be sure to always send a UNIX new-line character for EOL.
 *
 * Revision 1.2  2002-08-31 17:39:52-07  ball
 * Improved trace support.
 *
 * Revision 1.1  2002-08-30 18:50:30-07  ball
 * Initial revision
 *
 */
