/*
 * $Id: Board.java,v 1.6 2002-09-02 10:13:55-07 ball Exp $
 *
 * Copyright 2002 Allen D. Ball.  All rights reserved.
 */
package world;

import java.util.*;

/**
 * Class that implements the board for the ICFP-2002 programming contest
 * entry.
 *
 * @author <a href="mailto:ball@iprotium.com">Allen D. Ball</a>
 * @version $Revision: 1.6 $
 */
public class Board implements Direction {
    private int rows = 0;
    private int columns = 0;
    private Tile[][] tiles = null;
    private TreeMap map = new TreeMap();

    /**
     * Creates the board.
     *
     * @param   rows            The number of board rows.
     * @param   columns         The number of board columns.
     */
    public Board(int rows, int columns) {
        this.rows = rows;
        this.columns = columns;

        this.tiles = new Tile[rows][];
        for (int i = 0; i < tiles.length; i += 1) {
            tiles[i] = new Tile[columns];
        }
    }

    /**
     * Gets the board's row count.
     *
     * @return                  The number or rows.
     */
    public synchronized int getRowCount() { return rows; }

    /**
     * Gets the board's column count.
     *
     * @return                  The number or columns.
     */
    public synchronized int getColumnCount() { return columns; }

    /**
     * Gets the board's tiles.
     *
     * @param   row             The row position of the tile.
     * @param   column          The column position of the tile.
     *
     * @return                  The two-dimensional array of tiles.
     */
    public synchronized Tile[][] getTiles() { return tiles; }

    /**
     * Gets a tile on the board.
     *
     * @param   row             The row position of the tile.
     * @param   column          The column position of the tile.
     *
     * @return                  The tile at the specified position.
     */
    public synchronized Tile getTile(int row, int column) {
        return tiles[row][column];
    }

    /**
     * Sets a tile into the board.
     *
     * @param   tile            The tile to install.
     */
    public synchronized void setTile(Tile tile) {
        tiles[tile.getRow()][tile.getColumn()] = tile;
    }

    /**
     * Method to return the path from the start tile to the end tile.
     *
     * @param   start           The start tile of the path.
     * @param   end             The end tile of the path.
     *
     * @return                  The path from start to end.
     */
    public synchronized Path getPath(Tile start, Tile end) {
        Path path = (Path) map.get(new PathKey(start, end));

        if (path == null) {
            path = calculatePath(start, end);
            map.put(path, path);
        }

        return path;
    }

    private synchronized Path calculatePath(Tile start, Tile end) {
        Tile[] tiles = null;
        int[] steps = null;

        if (end.isPassable()) {
            Distances distances =
                new Distances(getRowCount(), getColumnCount());
            int distance = 0;
            TreeSet set = new TreeSet();

            set.add(start);
            while (distances.get(end) == 0 && set.size() > 0) {
                distance += 1;
                Tile[] edgeTiles = (Tile[]) set.toArray(new Tile[0]);
                set.clear();

                for (int i = 0; i < edgeTiles.length; i += 1) {
                    if (edgeTiles[i].isPassable()
                        && distances.get(edgeTiles[i]) < 1) {
                        distances.set(edgeTiles[i], distance);

                        set.add(edgeTiles[i].getNeighbor(NORTH));
                        set.add(edgeTiles[i].getNeighbor(SOUTH));
                        set.add(edgeTiles[i].getNeighbor(EAST));
                        set.add(edgeTiles[i].getNeighbor(WEST));
                    }
                }
            }

            /* Walk the results backwards to get the tiles and the steps
             * in the path.
             */
            if (distances.get(end) > 0) {
                tiles = new Tile[distances.get(end) - 1];
                steps = new int[distances.get(end) - 1];

                Tile tile = end;
                for (int i = steps.length - 1; i >= 0; i -= 1) {
                    tiles[i] = tile;

if (distances.get(tile.getNeighbor(NORTH)) == (distances.get(tile) - 1)) {
    steps[i] = SOUTH;
    tile = tile.getNeighbor(NORTH);
    continue;
}

if (distances.get(tile.getNeighbor(SOUTH)) == (distances.get(tile) - 1)) {
    steps[i] = NORTH;
    tile = tile.getNeighbor(SOUTH);
    continue;
}

if (distances.get(tile.getNeighbor(EAST)) == (distances.get(tile) - 1)) {
    steps[i] = WEST;
    tile = tile.getNeighbor(EAST);
    continue;
}

if (distances.get(tile.getNeighbor(WEST)) == (distances.get(tile) - 1)) {
    steps[i] = EAST;
    tile = tile.getNeighbor(WEST);
    continue;
}
                }
            }
        }

        return new Path(start, end, tiles, steps);
    }

    private class Distances {
        int [][] array = null;

        private Distances(int rows, int columns) {
            array = new int[rows][columns];
        }

        private int get(Tile tile) {
            return get(tile.getRow(), tile.getColumn());
        }

        private int get(int row, int column) {
            return array[row][column];
        }

        private void set(Tile tile, int value) {
            set(tile.getRow(), tile.getColumn(), value);
        }

        private void set(int row, int column, int value) {
            array[row][column] = value;
        }
    }
}
/*
 * $Log: Board.java,v $
 * Revision 1.6  2002-09-02 10:13:55-07  ball
 * Better use of sets.
 * Added trail to robots.
 * Optimized path calculation code (although there are still obvious
 * optimizations to be made).
 *
 * Revision 1.5  2002-09-02 03:22:25-07  ball
 * Store calculated paths for re-use.
 *
 * Revision 1.4  2002-09-01 23:38:55-07  ball
 * Combined Position class with Tile class.  Removed Position class.
 *
 * Revision 1.3  2002-09-01 15:10:58-07  ball
 * Tiles and Robots both use Sacks to hold packages.
 * Added standard Directions.  Added methods to Tile and Position
 * to take advantage of Directions.
 * GUI is no longer World-specific.
 *
 * Revision 1.2  2002-09-01 08:47:52-07  ball
 * Added position information to tiles.
 * Moved most diagnostic GUI code to World class.
 * Track player robot's actual and opponent robot's observed scores and
 * loads.  Track opponent robot's observed capacity.  Added methods to
 * manage, drop, and pick packages.
 * Add load and position columns to robot table.
 * Track game turn number.
 * Display robots on the world map.
 *
 * Revision 1.1  2002-08-31 17:52:00-07  ball
 * Initial revision
 *
 */
