/*
 * $Id: Robot.java,v 1.7 2002-09-02 10:13:55-07 ball Exp $
 *
 * Copyright 2002 Allen D. Ball.  All rights reserved.
 */
package world;

import java.util.*;

/**
 * Class that implements robots for the ICFP-2002 programming contest
 * entry.
 *
 * @author <a href="mailto:ball@iprotium.com">Allen D. Ball</a>
 * @version $Revision: 1.7 $
 */
public abstract class Robot implements ID, Locality {
    private World world = null;
    private int id = -1;
    private int score = 0;
    private int capacity = 0;
    private ArrayList trail = new ArrayList();
    private Sack sack = new Sack();
    private Tile tile = null;

    /**
     * Creates a robot.
     *
     * @param   world           The world this robot resides in.
     * @param   id              The robot ID.
     * @param   capacity        The robot's capacity.
     */
    protected Robot(World world, int id, int capacity) {
        this.world = world;
        this.id = id;
        this.capacity = capacity;
    }

    private Robot() { }

    /**
     * Method to get the robot's world of residence.
     *
     * @return                  The robot's world.
     */
    public synchronized World getWorld() { return world; }

    /**
     * Method to get the robot ID.
     *
     * @return                  The robot ID.
     */
    public synchronized int getId() { return id; }

    /**
     * Method to get the robot's capacity.
     *
     * @return                  The robot's capacity.
     */
    public synchronized int getCapacity() { return capacity; }

    /**
     * Method to set the robot's capacity.
     *
     * @param   capacity        The robot's capacity.
     */
    protected synchronized void setCapacity(int capacity) {
        this.capacity = capacity;
    }

    /**
     * Method to get the robot's current load.
     *
     * @return                  The robot's load.
     */
    public synchronized int getLoad() { return sack.getWeight(); }

    /**
     * Method to get the robot score.
     *
     * @return                  The robot score.
     */
    public synchronized int getScore() { return score; }

    /**
     * Method to set the robot score.
     *
     * @param   score           The robot score.
     */
    public synchronized void setScore(int score) { this.score = score; }

    /**
     * Method to get the tile's sack.
     *
     * @return                  The tile's sack.
     */
    public synchronized Sack getSack() { return sack; }

    /**
     * Method to get the robot's [location] tile.
     *
     * @return                  The robot's [location] tile.
     */
    public synchronized Tile getTile() { return tile; }

    /**
     * Method to set the robot's [location] tile.
     *
     * @return                  The robot's [location] tile.
     */
    public synchronized void setTile(Tile tile) { this.tile = tile; }

    /**
     * Method to move the robot North.
     */
    public synchronized void moveNorth() { move(Tile.NORTH); }

    /**
     * Method to move the robot South.
     */
    public synchronized void moveSouth() { move(Tile.SOUTH); }

    /**
     * Method to move the robot East.
     */
    public synchronized void moveEast() { move(Tile.EAST); }

    /**
     * Method to move the robot West.
     */
    public synchronized void moveWest() { move(Tile.WEST); }

    private void move(int direction) {
        setTile(tile.getNeighbor(direction));
        trail.add(tile);
    }

    public List getTrailList() { return trail; }

    public Tile[] getTrail() {
        return (Tile[]) getTrailList().toArray(new Tile[0]);
    }

    /**
     * Method to drop a package.
     *
     * @param   id              The package ID of the package to be dropped.
     */
    public synchronized void drop(int id) {
        Pkg pkg = sack.remove(id);

        /* This is likely to be a robot dropping a package at its
         * destination, so the player needs to be cautious...
         */
        if (pkg == null) {
            pkg = new Pkg(id, 0, null);
        }

        if (pkg != null) {
            if (pkg.getDestination() == tile) {
                setScore(getScore() + pkg.getWeight());
            } else {
                getTile().getSack().put(pkg);
            }
        }
    }

    /**
     * Method to pick[-up] a package.
     *
     * @param   id              The package ID of the package to be picked.
     */
    public synchronized void pick(int id) {
        Pkg pkg = (Pkg) getTile().getSack().remove(id);

        if (pkg == null) {
            pkg = new Pkg(id, 0, null);
        }

        sack.put(pkg);
    }

    /**
     * Method to determine if the robot is vulnerable.
     *
     * @return                  True if a another robot occupies a
     *                          neighboring space; false otherwise.
     */
    public boolean isVulnerable() {
        return (getTile().getNeighbor(Direction.NORTH).isLethal()
                || getTile().getNeighbor(Direction.SOUTH).isLethal()
                || getTile().getNeighbor(Direction.EAST).isLethal()
                || getTile().getNeighbor(Direction.WEST).isLethal());
    }

    /**
     * Method to get a string description of the robot.
     *
     * @return                  The string describing the robot.
     */
    public synchronized String toString() { return "Robot"; }

    /**
     * Compares this object with the specified object for order.
     * Returns a negative integer, zero, or a positive integer as this
     * object is less than, equal to, or greater than the specified
     * object.
     *
     * @param   object          The object to be compared.
     *
     * @see java.lang.Comparable
     */
    public synchronized int compareTo(Object object) {
        return getId() - ((Robot) object).getId();
    }
}
/*
 * $Log: Robot.java,v $
 * Revision 1.7  2002-09-02 10:13:55-07  ball
 * Better use of sets.
 * Added trail to robots.
 * Optimized path calculation code (although there are still obvious
 * optimizations to be made).
 *
 * Revision 1.6  2002-09-02 03:22:25-07  ball
 * Store calculated paths for re-use.
 *
 * Revision 1.5  2002-09-01 23:38:55-07  ball
 * Combined Position class with Tile class.  Removed Position class.
 *
 * Revision 1.4  2002-09-01 22:08:34-07  ball
 * Removed extra GUI support from IdMap and subclasses -- The GUI used in
 * development will be updated at the end of every turn.
 * The Position class implements Comparable.  Also added a toString()
 * method suitable for displaying the coordinates of the position.
 * Added a method to the Tile class for calculating the path between tiles.
 * Marked a water tile as "impassable."
 *
 * Revision 1.3  2002-09-01 15:10:58-07  ball
 * Tiles and Robots both use Sacks to hold packages.
 * Added standard Directions.  Added methods to Tile and Position
 * to take advantage of Directions.
 * GUI is no longer World-specific.
 *
 * Revision 1.2  2002-09-01 08:47:52-07  ball
 * Added position information to tiles.
 * Moved most diagnostic GUI code to World class.
 * Track player robot's actual and opponent robot's observed scores and
 * loads.  Track opponent robot's observed capacity.  Added methods to
 * manage, drop, and pick packages.
 * Add load and position columns to robot table.
 * Track game turn number.
 * Display robots on the world map.
 *
 * Revision 1.1  2002-08-31 17:52:00-07  ball
 * Initial revision
 *
 */
